from sqlalchemy import update
from sqlalchemy import desc
import pandas as pd
from pandas import DataFrame


def create(entity) -> None:
    from pagexdata.database import db_session
    db_session.add(entity)
    db_session.commit()


def create_all(entity_list) -> None:
    from pagexdata.database import db_session
    db_session.add_all(entity_list)
    db_session.commit()
    db_session.close()


def read_all(entity_class_):
    from pagexdata.database import db_session
    result = db_session.query(entity_class_).all()
    db_session.close()
    return result


def read_all_by(entity_class_, query_filter=None, query_order_by=None, order_by_desc=False):
    from pagexdata.database import db_session
    query = db_session.query(entity_class_)
    if query_filter is not None:
        query = query.filter(*query_filter)
    if query_order_by is not None and order_by_desc:
        query = query.order_by(query_order_by.desc())
    elif query_order_by is not None and not order_by_desc:
        query = query.order_by(query_order_by)
    result = query.all()
    db_session.close()
    return result


def read_all_as_dataframe(entity_class_) -> DataFrame:
    from pagexdata.database import url
    return pd.read_sql(entity_class_.__tablename__, url.render_as_string(hide_password=False))


def delete_all(entity_class_) -> None:
    from pagexdata.database import db_session
    result = db_session.query(entity_class_).all()
    for element in result:
        db_session.delete(element)
        db_session.commit()
        db_session.close()


def update_reddit_hits_ranking(date, subreddits, ticker_symbols, df, dataframe_reddithit_columns):
    from pagexdata.database import db_session
    from pagexdata.entities import RedditHits
    for subreddit in subreddits:
        for symbol in ticker_symbols:
            key = (df[dataframe_reddithit_columns.COLUMN_DATE] == date) & \
                  (df[dataframe_reddithit_columns.COLUMN_TICKER_SYMBOL] == symbol) & \
                  (df[dataframe_reddithit_columns.COLUMN_SUBREDDIT] == subreddit)
            db_session.execute(update(RedditHits)
                .where(RedditHits.date == date)
                .where(RedditHits.subreddit == subreddit)
                .where(RedditHits.ticker_symbol == symbol)
                .values(rank=int(df.loc[key][dataframe_reddithit_columns.COLUMN_RANK]),
                        previous_rank=int(df.loc[key][dataframe_reddithit_columns.COLUMN_PREVIOUS_RANK]),
                        change_rank=int(df.loc[key][dataframe_reddithit_columns.COLUMN_CHANGE_RANK]),
                        change_hits_one_day=int(df.loc[key][dataframe_reddithit_columns.COLUMN_CHANGE_HITS_1_DAY]),
                        change_hits_two_days=int(df.loc[key][dataframe_reddithit_columns.COLUMN_CHANGE_HITS_2_DAYS]),
                        change_hits_three_days=int(df.loc[key][dataframe_reddithit_columns.COLUMN_CHANGE_HITS_3_DAYS]),
                        change_hits_one_week=int(df.loc[key][dataframe_reddithit_columns.COLUMN_CHANGE_HITS_1_WEEK]),
                        change_hits_two_weeks=int(df.loc[key][dataframe_reddithit_columns.COLUMN_CHANGE_HITS_2_WEEKS]),
                        change_hits_four_weeks=int(df.loc[key][dataframe_reddithit_columns.COLUMN_CHANGE_HITS_4_WEEKS]),
                        hits_volatility_one_week=float(df.loc[key][dataframe_reddithit_columns.COLUMN_HITS_VOLATILITY_1_WEEK]),
                        hits_volatility_two_weeks=float(df.loc[key][dataframe_reddithit_columns.COLUMN_HITS_VOLATILITY_2_WEEKS])))
    db_session.commit()
    db_session.close()



import datetime

from sqlalchemy import Column, Integer, String, DateTime, Boolean, Float
from pagexdata.database import Base
from pydantic import BaseModel


class RedditHits(Base):
    __tablename__ = 'reddit_hits'

    date = Column(DateTime, primary_key=True)
    ticker_symbol = Column(String, primary_key=True)
    subreddit = Column(String, primary_key=True)
    hits = Column(Integer)
    positive_hits = Column(Boolean)
    rank = Column(Integer, nullable=True)
    previous_rank = Column(Integer, nullable=True)
    change_rank = Column(Integer, nullable=True)
    change_hits_one_day = Column(Integer, nullable=True)
    change_hits_two_days = Column(Integer, nullable=True)
    change_hits_three_days = Column(Integer, nullable=True)
    change_hits_one_week = Column(Integer, nullable=True)
    change_hits_two_weeks = Column(Integer, nullable=True)
    change_hits_four_weeks = Column(Integer, nullable=True)
    hits_volatility_one_week = Column(Float, nullable=True)
    hits_volatility_two_weeks = Column(Float, nullable=True)


class StockData(Base):
    __tablename__ = 'stockdata'

    date = Column(DateTime, primary_key=True)
    ticker_symbol = Column(String, primary_key=True)
    price = Column(Float)
    volume = Column(Integer)


class ImportLog(Base):
    __tablename__ = 'importlog'

    date = Column(DateTime, primary_key=True)
    type = Column(String, primary_key=True)
    state = Column(String)
    message = Column(String, nullable=True)
    extra = Column(String, nullable=True)


# pagination entities for rest api
class RedditHitsPage(BaseModel):
    date: datetime.datetime
    ticker_symbol: str
    subreddit: str
    hits: int
    rank: int
    previous_rank: int
    change_rank: int
    change_hits_one_day: int
    change_hits_two_days: int
    change_hits_three_days: int
    change_hits_one_week: int
    change_hits_two_weeks: int
    change_hits_four_weeks: int
    hits_volatility_one_week: float
    hits_volatility_two_weeks: float

    class Config:
        orm_mode = True


class StockDataPage(BaseModel):
    date: datetime.datetime
    ticker_symbol: str
    price: float
    volume: int

    class Config:
        orm_mode = True


class ImportLogPage(BaseModel):
    date: datetime.datetime
    type: str
    state: str
    message: str
    extra: str

    class Config:
        orm_mode = True
